import * as THREE from '../build/three.module.js';
import { OrbitControls } from './jsm/controls/OrbitControls.js';
import { CSS2DRenderer, CSS2DObject } from './jsm/renderers/CSS2DRenderer.js';

class App {
    constructor() {
        this._initialize();
    }

    _initialize() {
        this.domWebGL = document.createElement('div');
        document.body.appendChild(this.domWebGL);

        let scene = new THREE.Scene();
        scene.background = new THREE.Color(0x000000);

        let renderer = new THREE.WebGLRenderer( { antialias: true });
        this.domWebGL.appendChild(renderer.domElement);

        this.renderer = renderer;
        this.scene = scene;

        const labelRenderer = new CSS2DRenderer();
        labelRenderer.domElement.style.position = 'absolute';
        labelRenderer.domElement.style.top = '0px';
        this.domWebGL.appendChild(labelRenderer.domElement);
        this.labelRenderer = labelRenderer;

        this._clock = new THREE.Clock();

        this._setupCamera();
        this._setupModel();
        this._setupLights();
        this._setupControls();

        window.onresize = this.resize.bind(this);

        this.resize();
    }

    _setupControls() {
        const controls = new OrbitControls(this._camera, this.labelRenderer.domElement);
        controls.minDistance = 5;
        controls.maxDistance = 100;
    }

    _setupModel() {
        const EARTH_RADIUS = 1;
        const MOON_RADIUS = 0.27;

        const axesHelper = new THREE.AxesHelper(5);
        this.scene.add(axesHelper);

        const earthGeometry = new THREE.SphereGeometry(EARTH_RADIUS, 16, 16);
        const textureLoader = new THREE.TextureLoader();
        const earthMaterial = new THREE.MeshPhongMaterial({
            specular: 0x333333,
            shininess: 5,
            map: textureLoader.load('textures/planets/earth_atmos_2048.jpg'),
            specularMap: textureLoader.load( 'textures/planets/earth_specular_2048.jpg' ),
            normalMap: textureLoader.load( 'textures/planets/earth_normal_2048.jpg' ),
            normalScale: new THREE.Vector2( 0.85, 0.85 )
        });
        const earth = new THREE.Mesh(earthGeometry, earthMaterial);
        this.scene.add(earth);

        const moonGeometry = new THREE.SphereGeometry(MOON_RADIUS, 16, 16);
        const moonMaterial = new THREE.MeshPhongMaterial({
            shininess: 5,
            map: textureLoader.load('textures/planets/moon_1024.jpg')
        });
        const moon = new THREE.Mesh(moonGeometry, moonMaterial);
        this.scene.add(moon);

        const earthDiv = document.createElement('div');
        earthDiv.className = 'label';
        earthDiv.textContent = '지구 행성';
        earthDiv.style.marginTop = '-1em';
        const earthLabel = new CSS2DObject(earthDiv);
        earthLabel.position.set(0, EARTH_RADIUS, 0);
        earth.add(earthLabel);

        const moonDiv = document.createElement('div');
        moonDiv.className = 'label';
        moonDiv.textContent = '달 행성';
        moonDiv.style.marginTop = '-1em';
        const moonLabel = new CSS2DObject(moonDiv);
        moonLabel.position.set(0, MOON_RADIUS, 0);
        moon.add(moonLabel);

        this._moon = moon;
    }

    _setupLights() {
        const dirLight = new THREE.DirectionalLight(0xffffff);
        this._camera.add(dirLight);
    }

    _setupCamera() {
        const aspect = window.innerWidth / window.innerHeight;
   
        const camera = new THREE.PerspectiveCamera(45, aspect, 0.1, 200);
        camera.position.set(10, 5, 20);

        this.scene.add(camera);

        this._camera = camera;
    }

    update(time) {
        const elapsed = this._clock.getElapsedTime();

        this._moon.position.set(Math.sin(elapsed)*2, 0, Math.cos(elapsed)*2);
    }

    render(time) {
        requestAnimationFrame(this.render.bind(this));

        this.update(time);
        this.renderer.render(this.scene, this._camera);
        this.labelRenderer.render(this.scene, this._camera);
    }

    resize() {
        const aspect = window.innerWidth / window.innerHeight;

        this.renderer.setPixelRatio(window.devicePixelRatio);
        this.renderer.setSize(window.innerWidth, window.innerHeight);
        
        this.labelRenderer.setSize(window.innerWidth, window.innerHeight);

        this._camera.aspect = aspect;
        this._camera.updateProjectionMatrix();

    }
}

window.onload = function () {
    (new App()).render();
}